from io import StringIO

import qrcode as qr


class QRCode:
    def __init__(self, data):
        self.code = qr.QRCode()
        self.code.add_data(data)

    def __str__(self):
        handle = StringIO()
        self.code.print_ascii(handle)

        return handle.getvalue()



import click

from qrshare.network import Network


@click.command()
@click.argument('paths', nargs=-1, required=True)
@click.option('--port', default=4000, type=int, help='waitress server port')
def serve(paths, port):
    try:
        Network.serve(paths, port)
    except (FileNotFoundError, ValueError) as e:
        print(e)
        input('Press enter to exit')


if __name__ == '__main__':
    serve()



from uuid import uuid4

from pathlib import Path
from urllib.parse import quote
from socket import socket, AF_INET, SOCK_DGRAM
from typing import Tuple

from flask import Flask, render_template_string, send_file
from waitress import serve

from .qr import QRCode
from .template import shared_template


class Network:
    LOCALHOST_ADDRESS = "127.0.0.1"

    @staticmethod
    def app(paths: str, port: int) -> Tuple[Flask, str]:
        """
        :param path: absolute path to file you want to share
        :param port: port where server runs
        :return: app, url to file
        """
        local_ip = Network.local_ip()

        # create app
        app = Flask(__name__)

        shared = f'shared/{uuid4().fields[-1]}'

        # create file routes
        files = {}
        for path in paths:
            # ensure path exists and and is a file
            file = Path(path)
            if not file.exists():
                raise FileNotFoundError(f'"{path}" does not exist')
            elif not file.is_file():
                raise FileNotFoundError(f'"{path}" is not a file')

            # get absolute path to file
            file = file.resolve()

            # add route
            share_route = f'{shared}/{quote(file.name)}'
            files[file.name] = ({'file': file, 'route': share_route})

        @app.route(f'/{shared}')
        def share_home():
            return render_template_string(shared_template, links=files.values())

        @app.route(f'/{shared}/<filename>')
        def share_file(filename):
            return send_file(files[filename]['file'])

        if len(files) == 1:
            # directly download file
            share_url = f'{local_ip}:{port}/{list(files.values())[0]["route"]}'
        else:
            # go to home
            share_url = f'{local_ip}:{port}/{shared}'

        return app, share_url

    @staticmethod
    def serve(paths: str, port: int = 4000):
        """
        Creates and runs a waitress server where file[path] exposed

        :param path: absolute path to file you want to share
        :param port: port where server runs
        :return:
        """
        app, url = Network.app(paths, port)

        print(QRCode(url.encode('utf-8')))
        print("Scan the QR code above.")

        serve(app, port=port)

    @staticmethod
    def local_ip() -> str:
        local_socket = socket(AF_INET, SOCK_DGRAM)
        try:
            # doesn't even have to be reachable
            local_socket.connect(('10.255.255.255', 1))
            ip_address = local_socket.getsockname()[0]
        except:  # noqa: E722
            ip_address = Network.LOCALHOST_ADDRESS
        finally:
            local_socket.close()

        if ip_address == Network.LOCALHOST_ADDRESS:
            raise ConnectionError(
                "Could find valid IP address for a local network. "
                "Verify that you are connected to a router.")

        return ip_address



from setuptools import setup, find_packages

import qrshare

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='qrshare',
    version=qrshare.__version__,
    author="Schicksal",
    description="flask waitress file share program",
    author_email='mhaisham79@gmail.com',
    long_description=long_description,
    long_description_content_type='text/markdown',

    entry_points={
        "console_scripts": ["qrshare=qrshare.__main__:serve"]
    },
    install_requires=[
        'flask',
        'waitress',
        'qrcode',
        'click'
    ],

    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        "Operating System :: OS Independent",
    ],
    license="MIT license",
    keywords='qr share localhost',

    url='https://github.com/mHaisham/qrshare',
    project_urls={
        'Source code': 'https://github.com/mHaisham/qrshare'
    },
    packages=find_packages(),
    python_requires='>=3.6'
)



# using template string
# since change in working directory may cause templates folder to inaccesible
shared_template = """
<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <!-- <link
      rel="stylesheet"
      href="{{ url_for('static', filename='main.css') }}"
    /> -->
    <style>
      * {
        font-family: sans-serif;
        --text-primary: #cfcfcf;
        --text-secondary: #e9e9e9;
        --bg-primary: #333;
        --bg-secondary: #272727;
        --bg-accent: rgb(24, 24, 24);
      }

      body {
        background-color: var(--bg-primary);
        margin: 0;
      }

      h3 {
        margin: 0;
        padding: 1rem 0;
        color: var(--text-primary);
        background-color: var(--bg-accent);
        text-align: center;
      }

      ul {
        margin: 0;
        padding: 0;
        list-style: none;
      }

      a {
        display: block;
        text-decoration: none;
        padding: 1rem;
        text-align: center;
        color: var(--text-primary);
        background-color: var(--bg-primary);
        transition: 200ms ease-in;
      }

      li:nth-child(even) a {
        background-color: var(--bg-secondary);
      }

      a:hover {
        color: var(--text-secondary);
      }

      #da-btn {
        padding: 0.8rem;
        font-size: 0.8rem;
        font-weight: bold;
        background-color: var(--bg-accent);
      }
    </style>
    <title>QRShare</title>
  </head>
  <body>
    <div>
      <h3>Shared files</h3>
      <a href="#" id="da-btn">Download all</a>
      <ul>
        {% for link in links %}
        <li>
          <a href="/{{ link['route'] }}">
            {{ link['file'].name }}
          </a>
        </li>
        {% endfor %}
      </ul>
    </div>
  </body>
  <!-- <script src="{{ url_for('static', filename='main.js') }}"></script> -->
  <script defer>
    var links = document.getElementsByTagName("a");
    document.getElementById("da-btn").addEventListener("click", (e) => {
      e.preventDefault();

      for (let link of links) {
        link.click();
      }
    });
  </script>
</html>
"""
